package gov.lbl.kdd.data;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

public class GenReports {
	static class Stats {
		final int max;
		final int min;
		final int avg;
		final int total;
		final int count;
		final int httpErrors;
		final int callbackExceptions;
		final int inconsistent;
		
		public Stats(int max, int min, int avg, int total, int count, 
				int httpErrors, int callbackExceptions, int inconsistent) {
			this.max = max;
			this.min = min;
			this.avg = avg;
			this.total = total;
			this.count = count;
			this.httpErrors =httpErrors;
			this.callbackExceptions = callbackExceptions;
			this.inconsistent = inconsistent;
		}
		
		public String toString() {
			return "[MAX=" + max + ",MIN=" + min + ",AVG=" + avg 
					+ ",HTTP_ERROR_CODE=" + httpErrors + ",CALLBACK_EXCEPTIONS=" + callbackExceptions
					+ ",INCONSITENT=" + inconsistent +  "]";
		}
	}
	
	public static Stats computeStats(List<Stats> statsList) throws IOException {
		int max = Integer.MIN_VALUE;
		int min = Integer.MAX_VALUE;
		int total = 0;
		int count = 0;
		int httpErrors = 0;
		int callbackExceptions = 0;
		int inconsistent = 0;
		
		
		for (Stats stats: statsList) {
			max = Math.max(max, stats.max);
			min = Math.min(min, stats.min);
			total += stats.total;
			count += stats.count;
			httpErrors += stats.httpErrors;
			callbackExceptions += stats.callbackExceptions;
			inconsistent += stats.inconsistent;
		}
		
        return count != 0? new Stats(max, min, total/count, total, count, httpErrors, callbackExceptions, inconsistent) : 
        	new Stats(max, min, -1, total, count, httpErrors, callbackExceptions, inconsistent);
	}
	
	public static void dump(Map<String, List<Stats>> map, File reportFile) throws Exception {
		HSSFWorkbook wb = new HSSFWorkbook();
		FileOutputStream fileOut = new FileOutputStream(reportFile);
		
		try {
			List<Stats> summaryStatsList = new ArrayList<Stats>();

			{
				Iterator<Entry<String, List<Stats>>> iter2 = map.entrySet().iterator();
				HSSFSheet sheet = wb.createSheet("Musicians");

				// sheet.autoSizeColumn(1);
				// sheet.setDefaultColumnWidth(22);

				HSSFCellStyle style = wb.createCellStyle();

				style.setAlignment(HSSFCellStyle.ALIGN_RIGHT);

				sheet.setDefaultColumnStyle(0, style);
				sheet.setDefaultColumnStyle(1, style);
				sheet.setDefaultColumnStyle(2, style);
				sheet.setDefaultColumnStyle(3, style);
				sheet.setDefaultColumnStyle(4, style);
				sheet.setDefaultColumnStyle(5, style);
				sheet.setDefaultColumnStyle(6, style);
				sheet.setDefaultColumnStyle(7, style);
				sheet.setDefaultColumnStyle(8, style);

				HSSFRow headerRow = sheet.createRow(0);
				HSSFCell headerCell = headerRow.createCell(1);

				headerCell.setCellValue("MIN");
				headerCell = headerRow.createCell(2);
				headerCell.setCellValue("AVG");
				headerCell = headerRow.createCell(3);
				headerCell.setCellValue("MAX");
				headerCell = headerRow.createCell(4);
				headerCell.setCellValue("CALLBACK_EXCEPTIONS");
				headerCell = headerRow.createCell(5);
				headerCell.setCellValue("HTTP_ERROR_CODE");
				headerCell = headerRow.createCell(6);
				headerCell.setCellValue("INCONSISTENT");
				
				/*
				headerCell = headerRow.createCell(7);
				headerCell.setCellValue("COUNT");
				headerCell = headerRow.createCell(8);
				headerCell.setCellValue("TOTAL"); */

				int num = 1;

				while (iter2.hasNext()) {
					Map.Entry<String, List<Stats>> e = iter2.next();
					HSSFRow row = sheet.createRow(num++);
					Stats stats = computeStats(e.getValue());

					if (stats != null) {
						HSSFCell cell = row.createCell(0);
						cell.setCellValue(e.getKey());
						cell = row.createCell(1);
						cell.setCellValue(stats.min);
						cell = row.createCell(2);
						cell.setCellValue(stats.avg);
						cell = row.createCell(3);
						cell.setCellValue(stats.max);
						cell = row.createCell(4);
						cell.setCellValue(stats.callbackExceptions);
						cell = row.createCell(5);
						cell.setCellValue(stats.httpErrors);
						cell = row.createCell(6);
						cell.setCellValue(stats.inconsistent);
						
						/*
						cell = row.createCell(7);
						cell.setCellValue(stats.count);
						cell = row.createCell(8);
						cell.setCellValue(stats.total); */
						summaryStatsList.add(stats);
					}
				}

				Stats summaryStats = computeStats(summaryStatsList);

				if (summaryStats != null) {
					HSSFRow row = sheet.createRow(num);

					if (summaryStats != null) {
						HSSFCell cell = row.createCell(0);
						cell.setCellValue("Summary");
						cell = row.createCell(1);
						cell.setCellValue(summaryStats.min);
						cell = row.createCell(2);
						cell.setCellValue(summaryStats.avg);
						cell = row.createCell(3);
						cell.setCellValue(summaryStats.max);
						cell = row.createCell(4);
						cell.setCellValue(summaryStats.callbackExceptions);
						cell = row.createCell(5);
						cell.setCellValue(summaryStats.httpErrors);
						cell = row.createCell(6);
						cell.setCellValue(summaryStats.inconsistent);
						
						/*
						cell = row.createCell(7);
						cell.setCellValue(summaryStats.count);
						cell = row.createCell(8);
						cell.setCellValue(summaryStats.total); */
						
						System.out.println(summaryStats);
					}
				}
			}

			{
				Iterator<Entry<String, List<Stats>>> iter1 = map.entrySet()
						.iterator();

				while (iter1.hasNext()) {
					Map.Entry<String, List<Stats>> e = iter1.next();
					HSSFSheet sheet = wb.createSheet(e.getKey());
					HSSFCellStyle style = wb.createCellStyle();

					style.setAlignment(HSSFCellStyle.ALIGN_RIGHT);
					sheet.setDefaultColumnStyle(0, style);
					sheet.setDefaultColumnStyle(1, style);
					sheet.setDefaultColumnStyle(2, style);
					sheet.setDefaultColumnStyle(3, style);
					sheet.setDefaultColumnStyle(4, style);
					sheet.setDefaultColumnStyle(5, style);
					sheet.setDefaultColumnStyle(6, style);
					// sheet.setDefaultColumnStyle(7, style);
					// sheet.setDefaultColumnStyle(8, style);

					List<Stats> statsList = e.getValue();

					HSSFRow headerRow = sheet.createRow(0);
					HSSFCell headerCell = headerRow.createCell(1);

					headerCell.setCellValue("MIN");
					headerCell = headerRow.createCell(2);
					headerCell.setCellValue("AVG");
					headerCell = headerRow.createCell(3);
					headerCell.setCellValue("MAX");
					headerCell = headerRow.createCell(4);
					headerCell.setCellValue("CALLBACK_EXCEPTIONS");
					headerCell = headerRow.createCell(5);
					headerCell.setCellValue("HTTP_ERROR_CODE");
					headerCell = headerRow.createCell(6);
					headerCell.setCellValue("INCONSISTENT");
					
					/*
					headerCell = headerRow.createCell(7);
					headerCell.setCellValue("COUNT");
					headerCell = headerRow.createCell(8);
					headerCell.setCellValue("TOTAL"); */

					for (int i = 0; i < statsList.size(); ++i) {
						Stats stats = statsList.get(i);
						HSSFRow row = sheet.createRow(i + 1);

						HSSFCell cell = row.createCell(0);
						cell.setCellValue("RUN " + (i + 1));
						cell = row.createCell(1);

						cell.setCellValue(stats.min);
						cell = row.createCell(2);
						cell.setCellValue(stats.avg);
						cell = row.createCell(3);
						cell.setCellValue(stats.max);
						cell = row.createCell(4);
						cell.setCellValue(stats.callbackExceptions);
						cell = row.createCell(5);
						cell.setCellValue(stats.httpErrors);
						cell = row.createCell(6);
						cell.setCellValue(stats.inconsistent);
						
						/*
						cell = row.createCell(7);
						cell.setCellValue(stats.count);
						cell = row.createCell(8);
						cell.setCellValue(stats.total); */
					}
				}
			}

			wb.write(fileOut);
		} finally {
			fileOut.close();
		}
	}
	
	public static List<Stats> read(File logFile) throws IOException {
		List<Stats> statsList = new ArrayList<Stats>();
		DataInputStream in = new DataInputStream(new FileInputStream(logFile));
		BufferedReader br = new BufferedReader(new InputStreamReader(in));
		String strLine = null;

		int max = Integer.MIN_VALUE;
		int min = Integer.MAX_VALUE;
		int count = 0;
		int total = 0;
		int httpErrors = 0;
		int httpExceptions = 0;
		int inconsistent = 0;
		int avg = -1;

		String nonEndExperiment = null;

		while (nonEndExperiment != null || (strLine = br.readLine()) != null) {
			if (nonEndExperiment != null || strLine.indexOf("BEGIN_EXPERIMENT") != -1) {
				nonEndExperiment = null;
				
				while ((strLine = br.readLine()) != null &&  strLine.indexOf("END_EXPERIMENT") == -1) {
					if (strLine.indexOf("BEGIN_EXPERIMENT") != -1) {
						nonEndExperiment = "BEGIN_EXPERIMENT";
						break;
					}
					
					if (strLine.indexOf("ERROR_HTTP_CODE") != -1) {
						httpErrors++;
					} else if (strLine.indexOf("ERROR_CALLBACK_EXCEPTION")  != -1) {
						httpExceptions++;
					} else if (strLine.indexOf("ERROR_INCONSISTENT")  != -1) {
						inconsistent++;
					} else {
						int index = strLine.indexOf("PROCESSING_TIME=");
						
						if (index != -1) {
							String temp = strLine.substring(index + "PROCESSING_TIME=".length());
						
							int x = Integer.parseInt(temp);
							
							max = Math.max(max, x);
							min = Math.min(min, x);
							total += x;
							count++;
						}
					}
				}
				
				if (count != 0) {
					avg = total/count;
				}
				
				statsList.add(new Stats(max, min, avg, total, count, httpErrors, httpExceptions, inconsistent));
					
			    max = Integer.MIN_VALUE;
		        min = Integer.MAX_VALUE;
			    count = 0;
				total = 0;
				httpErrors = 0;
				httpExceptions = 0;
				inconsistent = 0;
				avg = -1;
			}
		}
		
		in.close();
		return statsList;
	}
	
	public static void main(String[] args) {
		System.setOut(System.err);

		if (args.length != 2) {
			System.err.println("Usage: gov.lbl.kdd.data.GenReports [logDir][reportFriendlyName");
			System.exit(1);
		}
		
		String logDirRep = args[0];
		File logDir = new File(logDirRep);
		
		
		if (logDir.isDirectory() == false) {
			System.err.println(logDirRep  + " is not a directory ..");
			System.exit(1);
		}
		
		String[] files = logDir.list();
		Map<String, List<Stats>> map = new TreeMap<String, List<Stats>>();
		
		for (String file :files) {
			if (file.indexOf("musician") != -1 && file.indexOf(".log") != -1) {
				String musician = file.substring(0, file.indexOf(".log"));
						
				try {
					List<Stats> statsList = read(new File(logDir, file));
					map.put(musician, statsList);
				} catch (IOException e) {
					e.printStackTrace(System.err);
				}
			}
		}
		
		try {
			File reportFile = new File(logDir, "report-" + args[1] + ".xls");
			dump(map, reportFile);
			System.out.println("Report saved to " + reportFile.getAbsolutePath());
		} catch (Exception e) {
			e.printStackTrace(System.err);
		}
	}
}
