package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.client.DocumentConsumer;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import org.apache.log4j.Logger;

public class DocumentConsumerImpl implements  DocumentConsumer  {
	private static Logger logger = Logger.getLogger(DocumentConsumerImpl.class);
	
	private int run;
	private int total;
	private int count;
	private int errors;
	private int inconsistent;
	private int exceptions;
	private Object listener;
	private final Map<String, Integer> firstDocumentSizeMap = Collections.synchronizedMap(new TreeMap<String, Integer>());
	private Map<String, Integer> documentSizeMap;
	private final String name;

	public DocumentConsumerImpl(String name, int total, Object listener) {
		this.name = name;
		this.total = total;
		this.listener = listener;
	}
	
	public void data(String arg0, String arg1, byte[] arg2) {
		String key = arg0 + ":" + arg1;
		
		logger.info("PERF:RUN=" + run + ":KEY=" + key + ":FILE_CHUNK_SIZE=" + arg2.length);
		
		if ("true".equalsIgnoreCase(System.getProperty("CHECK_CONSISTENCY_ACROSS_RUNS", "false"))) {
			if (documentSizeMap.get(key) == null) {
				documentSizeMap.put(key, Integer.valueOf(arg2.length));
		    } else {
		    	documentSizeMap.put(key, Integer.valueOf(arg2.length + documentSizeMap.get(key)));
		    }
		}
	}

	public void mimeType(String arg0, String arg1, String arg2) {
	}

	public void done(String arg0, String arg1) {
		String key = arg0 + ":" + arg1;
		
		logger.info("PERF:RUN="+ run + ":KEY="+ key 
				+ (documentSizeMap.get(key) == null? "NO document ..." : ":FILE_SIZE=" + documentSizeMap.get(key)) );
		
		try {
			if ("true".equalsIgnoreCase(System.getProperty(
					"CHECK_CONSISTENCY_ACROSS_RUNS", "false"))) {
				if (run == 1) {
					firstDocumentSizeMap.put(key, documentSizeMap.get(key));
				} else {
					Integer firstSize = firstDocumentSizeMap.get(key);
					Integer newSize = documentSizeMap.get(key);

				    if (firstSize == null && newSize != null) {
						logger.error("ERROR_INCONSISTENT:RUN=" + run + ":KEY="+ key + ":"
								+ firstSize + ":" + newSize);
						inconsistent++;
					} else if (firstSize != null && newSize == null) {
						logger.error("ERROR_INCONSISTENT:RUN" + run + ":KEY=" + key + ":"
								+ firstSize + ":" + newSize);
						inconsistent++;
					} else if (firstSize != null && firstSize.equals(newSize) == false) {
						logger.error("ERROR_INCONSISTENT:RUN" + run + ":KEY=" + key + ":"
								+ firstSize + ":" + newSize);
						inconsistent++;
					}
				}
			}
		} catch (Exception e) {
			logger.error("PERF:SEVERE:RUN=" + run, e);
		}

		synchronized(this) {
			count++;
			
			if (count == total) {
				if ("true".equalsIgnoreCase(System.getProperty("CHECK_CONSISTENCY_ACROSS_RUNS", "false"))) {
					PrintWriter writer = null;
					
					try {
						writer = new PrintWriter(new OutputStreamWriter(new FileOutputStream("results.log") ,"utf-8"));

						for (Map.Entry<String, Integer> e: firstDocumentSizeMap.entrySet()) {
							if (e.getKey().indexOf(arg0 + ":") == -1) {
								throw new RuntimeException("should never happen");
							}
							
							String docId = e.getKey().substring(e.getKey().indexOf(':') + 1);
							writer.print("DATASET_ID=");
							writer.print(arg0);
							writer.print(":DOC_ID=");
							writer.print(docId);
						
							
							if (e.getValue() != null) {
								writer.println(":FILE_SIZE=" + e.getValue());
							} else {
								writer.println(":FILE_SIZE=NONE (null)");
							}
						} 
						
						writer.flush();
					} catch(IOException ioe) {
						logger.error("PERF:IO:OCCURED WHEN DUMPING RESULTS");
					} catch(Exception e) {
						logger.error("PERF:SEVERE:OCCURED WHEN DUMPING RESULTS", e);
					} finally {
						if (writer != null) {
							writer.close();
							
							if (writer.checkError()) {
								logger.error("PERF:IOE:OCCURED WHEN DUMPING RESULTS");
							}
						}
					}
				}
				
				synchronized(listener) {
					listener.notify();
				}
			}
		}
	}

	public void exception(String arg0, String arg1, Exception arg2) {
		if (logger.isDebugEnabled()) {
			logger.debug("ERROR_CALLBACK_EXCEPTION:" + arg0 + ":" + arg1 + arg2.getMessage(), arg2);
		} else {
			logger.error("ERROR_CALLBACK_EXCEPTION:" + arg0 + ":" + arg1 + arg2.getMessage());
		}
		
		synchronized(this) {
			exceptions++;
		}
	}

	public void httpResponseCode(String arg0, String arg1, int arg2) {
		if (arg2 != 200) {
			logger.error("ERROR_HTTP_CODE:" + arg0 + ":" + arg1 + ":HTTP_CODE=" + arg2);
			
			synchronized(this) {
				errors++;
			}
		}
	}

	public boolean isDone() {
		return count == total;
	}
	
	public String toString() {
		return "NAME=" + name + " [COUNT=" + count + ":TOTAL=" + total 
				+ ":HTTP_CODE_NOT_OK=" + errors + ":INCONSISTENT=" + this.inconsistent + ":CALLBACK_EXCEPTIONS=" + exceptions + "]";
	}
	
	public void init() {
		count = 0;
		errors = 0;
		inconsistent = 0;
		exceptions = 0;
		documentSizeMap = Collections.synchronizedMap(new HashMap<String, Integer>());
		run++;
	}

	public int getRun() {
		return run;
	}
	
	public int getCount() {
		return count;
	}

	public int getErrors() {
		return errors;
	}


	public int getInconsistent() {
		return inconsistent;
	}

	public int getExceptions() {
		return exceptions;
	}
}
