package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.client.DataServiceClient;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetListResponse;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetListResponse.DataSetMetaData;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.Format;

import java.net.URI;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

public class DocumentImpl implements Tester {
	private static Logger logger = Logger.getLogger(DocumentImpl.class);
	
	private final String dataSetId;
	private final DocumentConsumerImpl consumer;
	private final String name;
	private final List<String> list;
	
	public DocumentImpl(String name, String dataSetId, 
			List<String> list) {
		this.name = name;
		this.dataSetId = dataSetId;
		this.list = list;
		consumer = new DocumentConsumerImpl(name, list.size(), this);
	}
	
	public void run() throws Exception {
		DataServiceClient client = null;
		
		consumer.init();
		
		logger.info("BEGIN_EXPERIMENT:RUN=" + consumer.getRun());
		
		try {
			String serverURI = System.getProperty("SERVER_URI", "http://localhost:8080/KDDDataService/resources");
			URI uri = new URI(serverURI);
			int numOfThreads = Integer.parseInt(System.getProperty("NUM_OF_THREADS", "1"));
			
			client = new DataServiceClient(uri, numOfThreads);
			
			
			boolean found = false;
			DataSetListResponse dsl = client.requestDataSetList(name);
			Iterator<DataSetMetaData> iter = dsl.iterator();
			
			while (iter.hasNext()) {
				DataSetMetaData dsmd = iter.next();
				String id = dsmd.getId();
				
				if (id.equals(dataSetId)) {
					logger.debug("Found " + dataSetId + ":" + dsmd.getName());
					found = true;
					break;
				}
			}
			
			if (!found) {
				logger.error("Did not Find " + dataSetId);
				throw new Exception("Did not find dataSetId " +  dataSetId);
			} 
			
			for (String id: list) {
				client.requestDocument(dataSetId,
						id, Format.Text,
						consumer, name);
			}
			
			long now = System.currentTimeMillis();
			
			synchronized(this) {
				while (consumer.isDone() == false) {
					this.wait(1000);
					logger.info(consumer);
				}
			}
			
			logger.info("PERF:" + dataSetId + ":PROCESSING_TIME=" + (System.currentTimeMillis() - now));
			logger.info("END_EXPERIMENT:RUN=" + + consumer.getRun());
		} catch(Exception e) {
			logger.error("Error", e);
			logger.info("END_EXPERIMENT:RUN=" + consumer.getRun() + ":" + e.getMessage());
			throw e;
		} finally {
			try {
				if (client != null) {
					client.shutdown();
				}
			} catch(Exception e) {
				logger.error("Error", e);
			}
		}
	}

	public DocumentConsumerImpl getConsumer() {
		return consumer;
	}
}
