package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest.QueryNode;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

/*
 * 744be3c5-077e-3030-92b3-81a13fbc8a7c:KDD-02-B-Treasury-SDN
   04908590-600c-32ed-9899-9040252a3fa6:KDD-02-B-TOSIG
   6563af14-8c1c-3168-aa32-7e61f9c2c1e1
   6563af14-8c1c-3168-aa32-7e61f9c2c1e1:KDD-02-B-VETTS
   13bcae25-363a-3709-b9f7-0fd95803d1d0:KDD-02-B-Harmony
   4a1b140d-36ac-3eef-a2b7-f15e289e956d:KDD-02-B-START-GTD
   a131fed5-bab1-3bf6-af68-088b6c684057:KDD-02-B-LDC-Gigaword
   01bace1d-e164-3f86-878e-cf06205e9099:KDD-02-B-NCTC-WITS
 */

public class Musician {
	private static Logger logger = Logger.getLogger(Musician.class);
	
	enum Operation {
		INIT("init"),
		RUN("run"),
		EXIT("exit"),
		UNSUPPORTED("unsupported");
		
		private Operation(String op) {
			this.op = op;
		}
		
		static Operation fromString(String operation) {
			if ("init".equalsIgnoreCase(operation)) {
				return INIT;
			} else if ("run".equalsIgnoreCase(operation)) {
				return RUN;
			} else if ("exit".equalsIgnoreCase(operation)) {
				return EXIT;
			} else {
				return UNSUPPORTED;
			}
		}
		
		@SuppressWarnings("unused")
		private String op;
	}
	
	private ServerSocket acceptSocket;
	private String name;
	
	public void init(int port) throws IOException {
		logger.info("Initializing musician with port " + port);
		acceptSocket = new ServerSocket(port);
		acceptSocket.setReuseAddress(true);
		name = java.net.InetAddress.getLocalHost().getHostName() + ":" + port;
	}
	
	public Tester initTester(String method, String dataSetId, String dictionary, int numRequests) throws Exception {
		Method m = Musician.class.getMethod("init" + method, String.class, String.class, Integer.class);
		
		return (Tester) m.invoke(this, dataSetId, dictionary, numRequests);
	}
	
	public void start() {
		Tester tester = null;
		
		while (true) {
			Socket conn;
			String line;
			OutputStream os;
			
			try {
				logger.debug("ACCEPTING connection ...");
				conn = acceptSocket.accept();
				logger.debug("GOT connection=" + conn);
			} catch(IOException e) {
				logger.error("ERROR occured while accepting connection ...", e);
				continue;
			}
			
			try {
				BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream()));
				
			    line = br.readLine();
				os = conn.getOutputStream();
			} catch(IOException e) {
				logger.error("ERROR occured while reading command ...", e);
				
				try {
					conn.close();
				} catch (IOException e1) {
				}
				
				conn = null;
				continue;
			}
			
			logger.debug("GOT COMMAND:" + line);
			String resp = "OK";
			StringTokenizer st = new StringTokenizer(line);
			Operation operation = null;
			
			if (st.hasMoreTokens()) {
				operation = Operation.fromString(st.nextToken());
					
				logger.debug("COMMAND=" + operation);
				
				switch(operation) {
				case INIT:
					try {
						String method = st.nextToken();
						String dataSetId = st.nextToken();
						String dictionary = st.nextToken();
						int numRequests = Integer.valueOf(st.nextToken());
						
						if (st.hasMoreTokens()) {
							String field = st.nextToken();
							Method m = Musician.class.getMethod("init" + method, String.class, String.class, Integer.class, String.class);
							
							tester = (Tester) m.invoke(this, dataSetId, dictionary, numRequests, field);
						} else {
							Method m = Musician.class.getMethod("init" + method, String.class, String.class, Integer.class);
							
							tester = (Tester) m.invoke(this, dataSetId, dictionary, numRequests);
						}
					} catch (Exception e) {
						logger.error("INIT", e);
						resp = "ERROR:COMMAND NOT SUPPORTED" + e.getMessage();
					} 
					
					break;
				case RUN:
					try {
						if (tester != null) {
							tester.run();
						} else {
							logger.info("Tester not initialized ..."); 
							resp = "INIT must first be called ...";
						}
					} catch (Exception e) {
						logger.error("RUN", e);
						resp = "ERROR:" + e.getMessage();
					}
					
					break;
				case EXIT:
					break;
				default:
					resp = "operation unsupported ...";
				}
			} 
			
			PrintWriter pw = new PrintWriter(os);
			
			if (operation != null) {
				pw.print(operation);
				pw.print(":");
				pw.println(resp);
			} else {
				pw.println("Must a provide operation [init|run|exit]");
			}
			
			pw.flush();
			
			if (pw.checkError()) {
				logger.error("Errored occured writing response to conductor ...");
			}
			
			if (conn != null) {
				try {
					conn.close();
				} catch (IOException e) {
				}
			}
			
			if (operation == Operation.EXIT) {
				System.exit(0);
			}
		}
	}
	
	private List<String> readFile(String fileName) throws Exception {
		InputStream is = null;
		
		if (new File(fileName).exists()) {
			is = new FileInputStream(fileName);
		} else {
			is = getClass().getResourceAsStream(fileName);
			
			if (is == null) {
				throw new IOException("unable to read " + fileName);
			}
		}
		
		try {
			return FileUtil.read(is);
		} finally {
			is.close();
		}
	}
	
	public DocumentImpl initTestDocument(String dataSetId, String docIdFileName, Integer size) throws Exception {
		List<String> docIds = readFile(docIdFileName);
		
		Set<String> set = new HashSet<String>(docIds);
		
		if (set.size() != docIds.size()) {
			throw new Exception("docIdFile contains duplicate document ids. This is not allowed.");
		}
		
		if (size >= 0) {
			List<String> temp = new ArrayList<String>();
			
			for (int i = 0; i < size; ++i) {
				temp.add(docIds.get(i));
			}
			
			docIds = temp;
		}
		
		return new DocumentImpl("DocumentTester@" + name, dataSetId, docIds);
	}
	
	public RecordCountImpl initTestRangeRecordCount(String dataSetId, String dictionaryName, Integer size, String field) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createRangeQueries(field, dictionary, (size < 0)? dictionary.size() : size);
		
		return new RecordCountImpl("RangeRecordCountTester@" + name, dataSetId, queries);
	}
	
	public DataImpl initTestRangeQuery(String dataSetId, String dictionaryName, Integer size, String field) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createRangeQueries(field, dictionary, (size < 0)? dictionary.size() : size);
		
		return new DataImpl("FuzzyQueryTester@" + name, dataSetId, null, queries);
	}
	
	public RecordCountImpl initTestFuzzyRecordCount(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createFuzzyQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new RecordCountImpl("FuzzyRecordCountTester@" + name, dataSetId, queries);
	}
	
	public DataImpl initTestFuzzyQuery(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createFuzzyQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new DataImpl("FuzzyQueryTester@" + name, dataSetId, null, queries);
	}
	
	public RecordCountImpl initTestWildcardRecordCount(String dataSetId, String dictinaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictinaryName);
		List<QueryNode> queries = QueryUtil.createWildcardQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new RecordCountImpl("WildcardRecordCountTester@" + name, dataSetId, queries);
	}
	
	public DataImpl initTestWildcardQuery(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createWildcardQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new DataImpl("WildcardQueryTester@" + name, dataSetId, null, queries);
	}
	
	public RecordCountImpl initTestKeywordRecordCount(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createSimpleQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new RecordCountImpl("KeywordRecordCountTester@" + name, dataSetId, queries);
	}
	
	
	public DataImpl initTestKeywordQuery(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);;
		List<QueryNode> queries = QueryUtil.createSimpleQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new DataImpl("KeywordQueryTester@" + name, dataSetId, null, queries);
	}
	
	public RecordCountImpl initTestAndRecordCount(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createAndQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new RecordCountImpl("AndRecordCountTester@" + name, dataSetId, queries);
	}
	
	public DataImpl initTestAndQuery(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createAndQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new DataImpl("AndQueryTester@" + name, dataSetId, null, queries);
	}
	
	public RecordCountImpl initTestOrRecordCount(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createOrQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new RecordCountImpl("OrRecordCountTester@" + name, dataSetId, queries);
	}
	
	
	public DataImpl initTestOrQuery(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createOrQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new DataImpl("OrQueryTester@" + name, dataSetId, null, queries);
	}
	
	public RecordCountImpl initTestNotRecordCount(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createNotQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new RecordCountImpl("NotRecordCountTester@" + name, dataSetId, queries);
	}
	
	public DataImpl initTestNotQuery(String dataSetId, String dictionaryName, Integer size) throws Exception {
		List<String> dictionary = readFile(dictionaryName);
		List<QueryNode> queries = QueryUtil.createNotQueries(dictionary, (size < 0)? dictionary.size() : size);
		
		return new DataImpl("NotQueryTester@" + name, dataSetId, null, queries);
	}
	
	public static void main(String[] args) {
		try {
			int port = Integer.valueOf(args[0]);
			InputStream is = null;
			
			if (new File("musician.properties").exists()) {
				is = new FileInputStream("musician.properties");
			} else {
				is = Musician.class.getResourceAsStream("musician.properties");
			}
			
			Properties props = new Properties();
			
			props.load(is);
			
			Set<Entry<Object, Object>> s =  props.entrySet();
			
			for (Entry<Object, Object> e : s) {
				System.setProperty((String) e.getKey(), (String) e.getValue());
				logger.info("SETTING PROPERTY:" + e.getKey() + "=" + e.getValue());
			}
			
			is.close();
			
			Musician musician = new Musician();
			musician.init(port);
			musician.start();
		} catch(Exception e) {
			logger.error("ERROR starting musician", e);
		}
	}
}
