package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.server.exceptions.KDDDataServiceException;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest.QueryNode;

import java.util.ArrayList;
import java.util.List;

public class QueryUtil {
	
	public static List<QueryNode> createRangeQueries(String field, List<String> dictionary, int numRequests) throws KDDDataServiceException {
		if (numRequests > dictionary.size()) {
			throw new IllegalArgumentException("numRequests > dictionary.size()");
		} else if (dictionary.size() == 0) {
			throw new IllegalArgumentException("empty dictionary");
		}
		
		List<QueryNode> queries = new ArrayList<QueryNode>();
		
		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createRangeNode(field, dictionary.get(0), kw);
				
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
	
	public static List<QueryNode> createOrQueries(List<String> dictionary, int numRequests) throws KDDDataServiceException  {
		if (numRequests > dictionary.size()) {
			throw new IllegalArgumentException("numRequests > dictionary.size()");
		} else if (dictionary.size() == 0) {
			throw new IllegalArgumentException("empty dictionary");
		}
		
		
		List<QueryNode> queries = new ArrayList<QueryNode>();
		QueryNode aQuery = QueryNode.createKeywordNode(dictionary.get(0));
		
		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createKeywordNode(kw);
				
				query = QueryNode.createOrNode(aQuery, query);
				
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
	
	public static List<QueryNode> createNotQueries(List<String> dictionary, int numRequests) throws KDDDataServiceException  {
		if (numRequests > dictionary.size()) {
			throw new IllegalArgumentException("numRequests > dictionary.size()");
		} else if (dictionary.size() == 0) {
			throw new IllegalArgumentException("empty dictionary");
		}
		
		List<QueryNode> queries = new ArrayList<QueryNode>();
		
		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createKeywordNode(kw);
				
				query = QueryNode.createNotNode(query);
				
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
	
	public static List<QueryNode> createAndQueries(List<String> dictionary, int numRequests) throws KDDDataServiceException  {
		if (numRequests > dictionary.size()) {
			throw new IllegalArgumentException("numRequests > dictionary.size()");
		} else if (dictionary.size() == 0) {
			throw new IllegalArgumentException("empty dictionary");
		}
		
		List<QueryNode> queries = new ArrayList<QueryNode>();
		QueryNode aQuery = QueryNode.createKeywordNode(dictionary.get(0));

		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createKeywordNode(kw);
				
				query = QueryNode.createAndNode(aQuery, query);
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
	
	public static List<QueryNode> createFuzzyQueries(List<String> dictionary, int numRequests) throws KDDDataServiceException {
		if (numRequests > dictionary.size()) {
			throw new IllegalArgumentException("numRequests > dictionary.size()");
		} else if (dictionary.size() == 0) {
			throw new IllegalArgumentException("empty dictionary");
		}
		
		List<QueryNode> queries = new ArrayList<QueryNode>();
		
		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createKeywordNode(kw + "~");
				query = QueryNode.createNotNode(query);
				
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
	
	public static List<QueryNode> createWildcardQueries(List<String> dictionary, int numRequests) throws KDDDataServiceException {
		if (numRequests > dictionary.size()) {
			throw new IllegalArgumentException("numRequests > dictionary.size()");
		} else if (dictionary.size() == 0) {
			throw new IllegalArgumentException("empty dictionary");
		}
		
		List<QueryNode> queries = new ArrayList<QueryNode>();
		
		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createKeywordNode("*" + kw + "*");
				
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
	
	
	public static List<QueryNode> createSimpleQueries(List<String> dictionary, Integer numRequests) throws KDDDataServiceException {
		if (numRequests > dictionary.size()) {
			throw new IllegalArgumentException("numRequests > dictionary.size()");
		} else if (dictionary.size() == 0) {
			throw new IllegalArgumentException("empty dictionary");
		}
		
		List<QueryNode> queries = new ArrayList<QueryNode>();
		
		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createKeywordNode(kw);
				
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
}
