package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.client.DataServiceClient;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetListResponse;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetListResponse.DataSetMetaData;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest.QueryNode;

import java.io.InputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

public class RecordCountImpl implements Tester {
	private static Logger logger = Logger.getLogger(RecordCountImpl.class);
	
	private final String dataSetId;
	private final RecordCountConsumerImpl consumer;
    private final String name;
	private final List<DataSetQueryRequest> list = new ArrayList<DataSetQueryRequest>();
	
	public RecordCountImpl(String name, String dataSetId, 
			List<QueryNode> queries) {
		this.name = name;
		this.dataSetId = dataSetId;
		int id = 0;
		int maxRecords = 100;
		
		if (System.getProperty("MAX_RECORDS") != null) {
			maxRecords = Integer.parseInt(System.getProperty("MAX_RECORDS"));
		}
		
		for (QueryNode query: queries) {
			DataSetQueryRequest dataSetQueryRequest = new DataSetQueryRequest(query);
			
			dataSetQueryRequest.setMaxRecords(maxRecords);
			
			String queryId = String.valueOf(id++);
			
			dataSetQueryRequest.setQueryId(queryId);
			list.add(dataSetQueryRequest);
		}
		
		consumer = new RecordCountConsumerImpl(name, list, this);
	}
	
	public void run() throws Exception {
		DataServiceClient client = null;
		
		consumer.init();
		
		logger.info("BEGIN_EXPERIMENT:RUN=" + consumer.getRun());
		
		try {
			String serverURI = System.getProperty("SERVER_URI", "http://localhost:8080/KDDDataService/resources");
			URI uri = new URI(serverURI);
			int numOfThreads = Integer.parseInt(System.getProperty("NUM_OF_THREADS", "1"));
			
			client = new DataServiceClient(uri, numOfThreads);
			
			boolean found = false;
			DataSetListResponse dsl = client.requestDataSetList(name);
			Iterator<DataSetMetaData> iter = dsl.iterator();
			
			while (iter.hasNext()) {
				DataSetMetaData dsmd = iter.next();
				String id = dsmd.getId();
				
				if (id.equals(dataSetId)) {
					logger.debug("Found " + dataSetId + ":" + dsmd.getName());
					found = true;
					break;
				}
			}
			
			if (!found) {
				logger.error("Did not Find " + dataSetId);
				throw new Exception("Did not find dataSetId " +  dataSetId);
			} 
			
			for (DataSetQueryRequest dataSetQueryRequest: list) {
				String queryId = dataSetQueryRequest.getQueryId();
				String temp = client.requestDataSetQueryRecordCount(dataSetId, dataSetQueryRequest, consumer, name);
				
				if (temp.equals(queryId) == false) {
					throw new RuntimeException(temp + ":" + queryId );
				} 
			}
			
			synchronized(this) {
				while (consumer.isDone() == false) {
					this.wait(1000);
					logger.info(consumer);
				}
			}
			
			logger.info("END_EXPERIMENT:RUN=" + consumer.getRun());
		} catch(Exception e) {
			logger.error("Error", e);
			logger.info("END_EXPERIMENT:RUN=" + consumer.getRun() + ":" + e.getMessage());
			throw e;
		} finally {
			try {
				if (client != null) {
					client.shutdown();
				}
			} catch(Exception e) {
				logger.error("Error", e);
			}
		}
	}
	
	public RecordCountConsumerImpl getConsumer() {
		return consumer;
	} 
	
	public static void main(String[] args) throws Exception {
		System.setProperty("SERVER_URI", "http://corona:8080/KDDDataService/resources");
		InputStream is = RecordCountImpl.class.getResourceAsStream("a131fed5-bab1-3bf6-af68-088b6c684057-keyword.txt");
		List<String> dictionary = FileUtil.read(is);
		
		is.close();
		
		int size = 10;
		
		List<QueryNode> queries = QueryUtil.createSimpleQueries(dictionary, (size < 0)? dictionary.size() : size);
		RecordCountImpl tester = new RecordCountImpl("KeywordTester", "a131fed5-bab1-3bf6-af68-088b6c684057", queries);
		
		tester.run();
		tester.run();
		tester.run();
	}
}
