package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.client.DataConsumer;
import gov.ornl.csed.csiir.idr.kdd.dataservice.client.DataServiceClient;
import gov.ornl.csed.csiir.idr.kdd.dataservice.client.RecordCountConsumer;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.exceptions.KDDDataServiceException;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest.QueryNode;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.RecordCountResponse;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.ResultSetResponse.ResultRecord;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.ResultSetResponse;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.ResultSetResponse.ResultRecord.ResultField;

import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;


public class BooleanQueryTest {
	private final static String[] dictionary = { 
		"Mel", "laden", "Anschutz", "Harvey", 
		"Aparna Mohan", "Dena Price", "Sal Mazzaro", 
		"President Bush", 
		"Middle East",
		"North Korea",
		"Musharraf",
		"Keith Vanderlaan",
		"Anthony M. Kennedy",
		"Tom DeLay",
		"Marta Lamas",
		"David Autor",
		"RHEUMATOID ARTHRITIS",
		"Washington Post",
		"Rick Owens",
		"Philadelphia",
		"Carla Sabloff",
		"Target",
		"University of Michigan",
		"Los Angeles Times",
		"WASHINGTON",
		"PARIS",
		"Saddam Hussein",
		"Milosevic",
		"Gabrielle",
		"Samuel L. Jackson",
		"Quincy Jones",
		"Russia",
		"Zola",
		"Oscar",
		"Horvitz",
		"Timothy Fagan",
		"Alan Tash",
		"FRESNO",
		"Civil War"
	};
	
	private URI uri;
	
	static class TestRecordCountConsumer implements RecordCountConsumer {
		private Set<String> doneSet;
		
		TestRecordCountConsumer(Set<String> doneSet) {
			this.doneSet = doneSet;
		}
		
		public void done(String arg0, String arg1) {
			synchronized(BooleanQueryTest.class) {
				doneSet.add(arg1);
				BooleanQueryTest.class.notify();
			}
		}

		public void exception(String arg0, String arg1, Exception arg2) {
			arg2.printStackTrace(System.err);
		}

		public void httpResponseCode(String arg0, String arg1, int arg2) {
			System.err.println("HTTP_RESPONSE:" + arg2);
		}

		public void recordCount(String arg0, String arg1,
				RecordCountResponse arg2) {
			System.out.println("MATCHING:" + arg2.getTotalMatchingRecords());;
		}
	} 
	 
	static class TestDataConsumer implements DataConsumer {
		private List<String> documentIds = new ArrayList<String>();
		
		private Set<String> doneSet;
		
		TestDataConsumer(Set<String> doneSet) {
			this.doneSet = doneSet;
		}
		
		public void data(String arg0, String arg1, ResultSetResponse arg2) {
			System.out.println(arg2.getQueryProcessingTime() + ":" + arg2.getColumnList().size());
			
			Iterator<ResultRecord> riter = arg2.iterator();
			
			while (riter.hasNext()) {
				ResultRecord rc = riter.next();
				
				Iterator<ResultField> iter2 = rc.iterator();
				
				while (iter2.hasNext()) {
					
				}
				
				String docId = rc.getString("id");
				
				documentIds.add(docId);
			}
		}

		public void done(String arg0, String arg1) {
			synchronized(BooleanQueryTest.class) {
				doneSet.add(arg1);
				BooleanQueryTest.class.notify();
			}
		}

		public void exception(String arg0, String arg1, Exception arg2) {
			arg2.printStackTrace(System.err);
		}

		public void httpResponseCode(String arg0, String arg1, int arg2) {
		}
	}
	
	@Before
	public void load() throws Exception {
		String server = System.getProperty("SERVER_NAME", "localhost");
		String port = System.getProperty("SERVER_PORT", "8080");
	
		uri = new URI("http://" + server + ":" + port + "/KDDDataService/resources");
	}

	
	public static QueryNode createOrQuery(String[] dictionary, int numRequests) throws KDDDataServiceException  {
		QueryNode query = QueryNode.createKeywordNode(dictionary[0]);
		
		for (String kw: dictionary) {
			QueryNode aQuery = QueryNode.createKeywordNode(kw);
			
			query = QueryNode.createOrNode(aQuery, query);

		}
		
		return query;
	}
	
	public static List<QueryNode> createOrQueries(String[] dictionary, int numRequests) throws KDDDataServiceException  {
		List<QueryNode> queries = new ArrayList<QueryNode>();
		QueryNode aQuery = QueryNode.createKeywordNode(dictionary[0]);
		
		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createKeywordNode(kw);
				
				for (int i = 0; i < 100; ++i) {
					query = QueryNode.createOrNode(aQuery, query);
				}
				
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
	
	public static List<QueryNode> createAndQueries(String[] dictionary, int numRequests) throws KDDDataServiceException  {
		List<QueryNode> queries = new ArrayList<QueryNode>();
		QueryNode aQuery = QueryNode.createKeywordNode(dictionary[0]);
		
		do {
			for (String kw: dictionary) {
				QueryNode query = QueryNode.createKeywordNode(kw);
				
				for (int i = 0; i < 100; ++i) {
					query = QueryNode.createAndNode(aQuery, query);
				}
				
				queries.add(query);
				
				if (queries.size() == numRequests) {
					break;
				}
			}
		} while (queries.size() < numRequests);
		
		return queries;
	}
	
	
	// @Test
	public void test1() throws Exception {
		Set<String> doneSet = new TreeSet<String>();
		// URI uri = new URI("http://corona.lbl.gov:8080/KDDDataService/resources");
		DataServiceClient client = new DataServiceClient(uri, 10); 
		QueryNode query = createOrQuery(dictionary, 1);;
		
		List<String> queryIds = new ArrayList<String>();
		
		for (int i = 0; i < 1; ++i) {
			DataSetQueryRequest dataSetQueryRequest = new DataSetQueryRequest(query);
			String queryId = client.requestDataSetQuery("744be3c5-077e-3030-92b3-81a13fbc8a7c", dataSetQueryRequest, 
					new TestDataConsumer(doneSet), null);
			
			queryIds.add(queryId );
		}
		
		synchronized(BooleanQueryTest.class) {
			while (doneSet.size() != queryIds.size()) {
				BooleanQueryTest.class.wait(100 * 1000);
				System.out.println("STILL WAITING ...");
			}
		}
		
		client.shutdown();
	}
	
	

	
	// @Test
	public void test2() throws Exception {
		Set<String> doneSet = new TreeSet<String>();
		// URI uri = new URI("http://corona.lbl.gov:8080/KDDDataService/resources");
		DataServiceClient client = new DataServiceClient(uri, 10); 
		List<QueryNode> requests = createAndQueries(dictionary, 1);
		QueryNode query = requests.get(0);
		
		List<String> queryIds = new ArrayList<String>();
		
		for (int i = 0; i < 1; ++i) {
			DataSetQueryRequest dataSetQueryRequest = new DataSetQueryRequest(query);
			String queryId = client.requestDataSetQuery("744be3c5-077e-3030-92b3-81a13fbc8a7c", dataSetQueryRequest, 
					new TestDataConsumer(doneSet), null);
			
			queryIds.add(queryId );
		}
		
		synchronized(BooleanQueryTest.class) {
			while (doneSet.size() != queryIds.size()) {
				BooleanQueryTest.class.wait(100 * 1000);
				System.out.println("STILL WAITING ...");
			}
		}
		
		client.shutdown();
	}
	
	// @Test
	public void test3() throws Exception {
		Set<String> doneSet = new TreeSet<String>();
		// URI uri = new URI("http://corona.lbl.gov:8080/KDDDataService/resources");
		DataServiceClient client = new DataServiceClient(uri, 10); 
		QueryNode query = QueryNode.createKeywordNode("*");
		List<String> queryIds = new ArrayList<String>();
		TestDataConsumer consumer = new TestDataConsumer(doneSet); 
		
		for (int i = 0; i < 1; ++i) {
			DataSetQueryRequest dataSetQueryRequest = new DataSetQueryRequest(query);
			
			dataSetQueryRequest.setMaxRecords(1000);
			String queryId = client.requestDataSetQuery("a131fed5-bab1-3bf6-af68-088b6c684057", dataSetQueryRequest, 
					consumer, null);
			
			queryIds.add(queryId);
		}
		
		synchronized(BooleanQueryTest.class) {
			while (doneSet.size() != queryIds.size()) {
				System.out.println("WILDCARD STILL WAITING ...");
				BooleanQueryTest.class.wait(100 * 1000);
				System.out.println("WILDCARD STILL WAITING ...");
			}
		}
		System.out.println("HI:" + consumer.documentIds.size() + consumer.documentIds);
		client.shutdown();
	}
	
	@Test
	public void test4() throws Exception {
		Set<String> doneSet = new TreeSet<String>();
		// URI uri = new URI("http://corona.lbl.gov:8080/KDDDataService/resources");
		DataServiceClient client = new DataServiceClient(uri, 10); 
		QueryNode query = QueryNode.createKeywordNode("*");
		List<String> queryIds = new ArrayList<String>();
		TestRecordCountConsumer consumer = new TestRecordCountConsumer(doneSet); 
		
		for (int i = 0; i < 1; ++i) {
			DataSetQueryRequest dataSetQueryRequest = new DataSetQueryRequest(query);
			
			dataSetQueryRequest.setMaxRecords(9849381);
			String queryId = client.requestDataSetQueryRecordCount("a131fed5-bab1-3bf6-af68-088b6c684057", dataSetQueryRequest, 
					consumer, null);
			
			queryIds.add(queryId);
		}
		
		synchronized(BooleanQueryTest.class) {
			while (doneSet.size() != queryIds.size()) {
				System.out.println("WILDCARD STILL WAITING ...");
				BooleanQueryTest.class.wait(100 * 1000);
				System.out.println("WILDCARD STILL WAITING ...");
			}
		}
		
		client.shutdown();
	}
	
	@After
	public void clear() throws Exception {
	}
}
