package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest.QueryNode;

import java.io.InputStream;
import java.util.List;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;


public class RecordCountTest {
	
	private List<String> dictionary;
	private int numRequests;
	private int numRuns;

	@Before
	public void load() throws Exception {
		InputStream is = getClass().getResourceAsStream("a131fed5-bab1-3bf6-af68-088b6c684057-keyword.txt");
		dictionary = FileUtil.read(is);
		
		is.close();
		
		numRequests = Integer.parseInt(System.getProperty("NUM_OF_REQUESTS", "1"));
		numRuns = Integer.parseInt(System.getProperty("NUM_OF_RUNS", "2"));
	}
	
	@Test
	public void test1() throws Exception {
		List<QueryNode> queries = QueryUtil.createSimpleQueries(dictionary, (numRequests < 0)? dictionary.size() : numRequests);
		RecordCountImpl tester = new RecordCountImpl("RecordCountTester", "a131fed5-bab1-3bf6-af68-088b6c684057", queries);
		
		System.out.println("TESTING  SIMPLE QUERIES ...");
		
		for (int i = 0; i < numRuns; ++i) {
			System.out.println("BEGIN RUN " + (i + 1));
			tester.run();
			
			Assert.assertTrue("INCONSISTENT", Integer.valueOf(tester.getConsumer().getInconsistent()) == 0);
			Assert.assertTrue("ERRORS", tester.getConsumer().getErrors() == 0);
			Assert.assertTrue("EXCEPTIONS", tester.getConsumer().getExceptions() == 0);
			Assert.assertTrue("COUNT", tester.getConsumer().getCount() == numRequests);
			System.out.println("END RUN " + (i + 1));
		}
	}
	
	@Test
	public void test2() throws Exception {
		List<QueryNode> queries = QueryUtil.createAndQueries(dictionary, (numRequests < 0)? dictionary.size() : numRequests);
		RecordCountImpl tester = new RecordCountImpl("RecordCountTester", "a131fed5-bab1-3bf6-af68-088b6c684057", queries);
		
		System.out.println("TESTING AND QUERIES ...");
		
		for (int i = 0; i < numRuns; ++i) {
			System.out.println("BEGIN RUN " + (i + 1));
			tester.run();
			
			Assert.assertTrue("INCONSISTENT", Integer.valueOf(tester.getConsumer().getInconsistent()) == 0);
			Assert.assertTrue("ERRORS", tester.getConsumer().getErrors() == 0);
			Assert.assertTrue("EXCEPTIONS", tester.getConsumer().getExceptions() == 0);
			Assert.assertTrue("COUNT", tester.getConsumer().getCount() == numRequests);
			System.out.println("END RUN " + (i + 1));
		}
	}
	
	@Test
	public void test3() throws Exception {
		List<QueryNode> queries = QueryUtil.createOrQueries(dictionary, (numRequests < 0)? dictionary.size() : numRequests);
		RecordCountImpl tester = new RecordCountImpl("RecordCountTester", "a131fed5-bab1-3bf6-af68-088b6c684057", queries);
		
		System.out.println("TESTING OR QUERIES ...");
		
		for (int i = 0; i < numRuns; ++i) {
			System.out.println("BEGIN RUN " + (i + 1));
			tester.run();
			
			Assert.assertTrue("INCONSISTENT", Integer.valueOf(tester.getConsumer().getInconsistent()) == 0);
			Assert.assertTrue("ERRORS", tester.getConsumer().getErrors() == 0);
			Assert.assertTrue("EXCEPTIONS", tester.getConsumer().getExceptions() == 0);
			Assert.assertTrue("COUNT", tester.getConsumer().getCount() == numRequests);
			System.out.println("END RUN " + (i + 1));
		}
	}
	
	@Test
	public void test4() throws Exception {
		List<QueryNode> queries = QueryUtil.createNotQueries(dictionary, (numRequests < 0)? dictionary.size() : numRequests);
		RecordCountImpl tester = new RecordCountImpl("RecordCountTester", "a131fed5-bab1-3bf6-af68-088b6c684057", queries);
		
		System.out.println("TESTING NOT QUERIES ...");
		
		for (int i = 0; i < numRuns; ++i) {
			System.out.println("BEGIN RUN " + (i + 1));
			tester.run();
			
			Assert.assertTrue("INCONSISTENT", Integer.valueOf(tester.getConsumer().getInconsistent()) == 0);
			Assert.assertTrue("ERRORS", tester.getConsumer().getErrors() == 0);
			Assert.assertTrue("EXCEPTIONS", tester.getConsumer().getExceptions() == 0);
			Assert.assertTrue("COUNT", tester.getConsumer().getCount() == numRequests);
			System.out.println("END RUN " + (i + 1));
		}
	}
	
	@Test
	public void test5() throws Exception {
		List<QueryNode> queries = QueryUtil.createFuzzyQueries(dictionary, (numRequests < 0)? dictionary.size() : numRequests);
		RecordCountImpl tester = new RecordCountImpl("RecordCountTester", "a131fed5-bab1-3bf6-af68-088b6c684057", queries);
		
		System.out.println("TESTING FUZZY QUERIES ...");
		
		
		for (int i = 0; i < numRuns; ++i) {
			System.out.println("BEGIN RUN " + (i + 1));
			tester.run();
			
			Assert.assertTrue("INCONSISTENT", Integer.valueOf(tester.getConsumer().getInconsistent()) == 0);
			Assert.assertTrue("ERRORS", tester.getConsumer().getErrors() == 0);
			Assert.assertTrue("EXCEPTIONS", tester.getConsumer().getExceptions() == 0);
			Assert.assertTrue("COUNT", tester.getConsumer().getCount() == numRequests);
			System.out.println("END RUN " + (i + 1));
		}
	}

	@Test
	public void test6() throws Exception {
		List<QueryNode> queries = QueryUtil.createWildcardQueries(dictionary, (numRequests < 0)? dictionary.size() : numRequests);
		RecordCountImpl tester = new RecordCountImpl("RecordCountTester", "a131fed5-bab1-3bf6-af68-088b6c684057", queries);
		
		System.out.println("TESTING WILDCARD QUERIES ...");
		
		
		for (int i = 0; i < numRuns; ++i) {
			System.out.println("BEGIN RUN " + (i + 1));
			tester.run();
			
			Assert.assertTrue("INCONSISTENT", Integer.valueOf(tester.getConsumer().getInconsistent()) == 0);
			Assert.assertTrue("ERRORS", tester.getConsumer().getErrors() == 0);
			Assert.assertTrue("EXCEPTIONS", tester.getConsumer().getExceptions() == 0);
			Assert.assertTrue("COUNT", tester.getConsumer().getCount() == numRequests);
			System.out.println("END RUN " + (i + 1));
		}
	}

	
	@After
	public void clear() throws Exception {
	}
}
