package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.client.DataConsumer;
import gov.ornl.csed.csiir.idr.kdd.dataservice.client.DataServiceClient;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest.QueryNode;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.ResultSetResponse;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;


public class ShutdownTest {
	URI uri;
	
	static class TestDataConsumer implements DataConsumer {
		private Set<String> doneSet;
		
		TestDataConsumer(Set<String> doneSet) {
			this.doneSet = doneSet;
		}
		
		public void data(String arg0, String arg1, ResultSetResponse arg2) {
		}

		public void done(String arg0, String arg1) {
			synchronized(ShutdownTest.class) {
				doneSet.add(arg1);
				ShutdownTest.class.notify();
			}
		}

		public void exception(String arg0, String arg1, Exception arg2) {
			arg2.printStackTrace(System.err);
		}

		public void httpResponseCode(String arg0, String arg1, int arg2) {
		}
	}
	
	@Before
	public void load() throws Exception {
		String server = System.getProperty("SERVER_NAME", "localhost");
		String port = System.getProperty("SERVER_PORT", "8080");
	
		uri = new URI("http://" + server + ":" + port + "/KDDDataService/resources");
	}
	
	@Test
	public void test1() throws Exception {
		Set<String> doneSet = new TreeSet<String>();
		// URI uri = new URI("http://corona.lbl.gov:8080/KDDDataService/resources");
		DataServiceClient client = new DataServiceClient(uri, 10); 
		QueryNode query = QueryNode.createRangeNode("uid", "0", "10000");
		
		List<String> queryIds = new ArrayList<String>();
		
		for (int i = 0; i < 100; ++i) {
			DataSetQueryRequest dataSetQueryRequest = new DataSetQueryRequest(query);
			String queryId = client.requestDataSetQuery("744be3c5-077e-3030-92b3-81a13fbc8a7c", dataSetQueryRequest, 
					new TestDataConsumer(doneSet), null);
			
			queryIds.add(queryId );
		}
		
		Thread.sleep(20000);
		
		Set<String> cancelledSet = new TreeSet<String>();
		
		for (String queryId: queryIds) {
			boolean ret = client.requestCancel(queryId);
			
			if (ret) {
				cancelledSet.add(queryId);
			}
		}
	
		client.shutdown();
		
		synchronized(ShutdownTest.class) {
			while (doneSet.size() + cancelledSet.size() < queryIds.size()) {
				ShutdownTest.class.wait(100 * 1000);
				System.out.println(cancelledSet.size() + ":" + doneSet.size());
			}
		}
		
		Assert.assertEquals(cancelledSet.size() + doneSet.size(), queryIds.size());
		System.out.println("EXITING:" + cancelledSet.size() + ":" + doneSet.size());
	}
	
	@After
	public void clear() throws Exception {
	}
}
