package gov.lbl.kdd.data;

import gov.ornl.csed.csiir.idr.kdd.dataservice.client.DataServiceClient;
import gov.ornl.csed.csiir.idr.kdd.dataservice.client.RecordCountConsumer;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.DataSetQueryRequest.QueryNode;
import gov.ornl.csed.csiir.idr.kdd.dataservice.server.models.RecordCountResponse;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;


public class WildQueryTest {
	private URI uri;
	
	static class TestRecordCountConsumer implements RecordCountConsumer {
		private Set<String> doneSet;
		
		TestRecordCountConsumer(Set<String> doneSet) {
			this.doneSet = doneSet;
		}
		
		public void done(String arg0, String arg1) {
			synchronized(WildQueryTest.class) {
				doneSet.add(arg1);
				WildQueryTest.class.notify();
			}
		}

		public void exception(String arg0, String arg1, Exception arg2) {
			arg2.printStackTrace(System.err);
		}

		public void httpResponseCode(String arg0, String arg1, int arg2) {
			System.err.println("HTTP_RESPONSE:" + arg2);
		}

		private long recordCount;
		
		public void recordCount(String arg0, String arg1,
				RecordCountResponse arg2) {
			recordCount = arg2.getTotalMatchingRecords();
		}

		public long getRecordCount() {
			return recordCount;
		}
	} 
	 
	
	@Before
	public void load() throws Exception {
		String serverURI = System.getProperty("SERVER_URI", "http://localhost:8080/KDDDataService/resources");
		uri = new URI(serverURI);
	}
	
	@Test
	public void test() throws Exception {
		Set<String> doneSet = new TreeSet<String>();
		DataServiceClient client = new DataServiceClient(uri, 1); 
		QueryNode query = QueryNode.createKeywordNode("*");
		List<String> queryIds = new ArrayList<String>();
		TestRecordCountConsumer consumer = new TestRecordCountConsumer(doneSet); 
		
		for (int i = 0; i < 1; ++i) {
			DataSetQueryRequest dataSetQueryRequest = new DataSetQueryRequest(query);
			
			dataSetQueryRequest.setMaxRecords(9849381);
			String queryId = client.requestDataSetQueryRecordCount("a131fed5-bab1-3bf6-af68-088b6c684057", dataSetQueryRequest, 
					consumer, null);
			
			queryIds.add(queryId);
		}
		
		synchronized(WildQueryTest.class) {
			while (doneSet.size() != queryIds.size()) {
				System.out.println("WILDCARD STILL WAITING ...");
				WildQueryTest.class.wait(100 * 1000);
			}
		}
		
		Assert.assertTrue("MATCHING", 9849381 == consumer.getRecordCount());
		client.shutdown();
	}
	
	@After
	public void clear() throws Exception {
	}
}
